@@ -3,7 +3,6 @@
 # Runs a subsetting test suite. Compares the results of subsetting via harfbuzz
 # to subsetting via fonttools.
 
-import io
 from difflib import unified_diff
 import os
 import re
@@ -26,7 +25,7 @@ def cmd (command):
 		command, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
 		universal_newlines=True)
 	(stdoutdata, stderrdata) = p.communicate ()
-	print (stderrdata, end="") # file=sys.stderr
+	print (stderrdata, end="", file=sys.stderr)
 	return stdoutdata, p.returncode
 
 def read_binary (file_path):
@@ -74,9 +73,9 @@ def run_test (test, should_check_ots):
 		if os.path.exists (actual_ttx): os.remove (actual_ttx)
 		return fail_test (test, cli_args, "ttx (actual) returned %d" % (return_code))
 
-	with io.open (expected_ttx, encoding='utf-8') as f:
+	with open (expected_ttx, encoding='utf-8') as f:
 		expected_ttx_text = f.read ()
-	with io.open (actual_ttx, encoding='utf-8') as f:
+	with open (actual_ttx, encoding='utf-8') as f:
 		actual_ttx_text = f.read ()
 
 	# cleanup
@@ -121,32 +120,29 @@ def has_ots ():
 def check_ots (path):
 	ots_report, returncode = cmd ([ots_sanitize, path])
 	if returncode:
-		print("OTS Failure: %s" % ots_report);
+		print ("OTS Failure: %s" % ots_report)
 		return False
 	return True
 
 args = sys.argv[1:]
 if not args or sys.argv[1].find ('hb-subset') == -1 or not os.path.exists (sys.argv[1]):
-	print ("First argument does not seem to point to usable hb-subset.")
-	sys.exit (1)
+	sys.exit ("First argument does not seem to point to usable hb-subset.")
 hb_subset, args = args[0], args[1:]
 
 if not len (args):
-	print ("No tests supplied.")
-	sys.exit (1)
+	sys.exit ("No tests supplied.")
 
 has_ots = has_ots()
 
 fails = 0
 for path in args:
-	with io.open (path, mode="r", encoding="utf-8") as f:
+	with open (path, mode="r", encoding="utf-8") as f:
 		print ("Running tests in " + path)
 		test_suite = SubsetTestSuite (path, f.read ())
 		for test in test_suite.tests ():
 			fails += run_test (test, has_ots)
 
 if fails != 0:
-	print (str (fails) + " test(s) failed.")
-	sys.exit(1)
+	sys.exit ("%d test(s) failed." % fails)
 else:
 	print ("All tests passed.")
